==> spring-boot-ext-1.2.298-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/replay/ReplayTraceLoader.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay.replay;

import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayFileComponent;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayTrace;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.event.ReplayEndEvent;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.event.ReplayStartEvent;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.serializer.Hessian2Serializer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * 下载并解压录制的用例集文件，然后把用例读取到内存中，供用例消费者消费
 * Created by guoyanfei .
 * 2019-02-26 .
 */
@Slf4j
public class ReplayTraceLoader {

    @Resource
    private FlowReplayFileComponent flowReplayFileComponent;

    private volatile Thread usecaseSetLoadThread = null;

    /**
     * 监听 "回放开始" 事件，加载用例集文件内容到内存中
     * @param event
     */
    @EventListener(ReplayStartEvent.class)
    public void replayStartEventListener(ReplayStartEvent event) {
        usecaseSetLoadThread = new Thread(new Runnable() {

            @Override
            public void run() {
                ReplayContext context = ReplayContextHolder.getReplayContext();
                if (context == null) {
                    return;
                }
                try {
                    loadUsecaseSet(context);
                    // 用例集加载完成
                    context.tracesLoadFinish();
                } catch (Exception e) {
                    log.error("引流回归_用例集加载异常_reportId_{}", context.getReportId(), e);
                    ReplayContextHolder.forceEnd();
                }
            }
        }, "DBThread-Replay-加载用例集");

        usecaseSetLoadThread.start();
    }

    @EventListener(ReplayEndEvent.class)
    public void replayEndEventListener(ReplayEndEvent event) {
        if (usecaseSetLoadThread != null) {
            usecaseSetLoadThread.interrupt();
        }
    }

    /**
     * 加载用例集
     * @throws IOException
     */
    private void loadUsecaseSet(ReplayContext context) throws IOException {
        String localReplayPath = flowReplayFileComponent.download(context.getUsecaseSetUrl());
        File file = new File(localReplayPath);
        try (RandomAccessFile raf = new RandomAccessFile(file, "rw")) {
            raf.skipBytes(8);   // 跳过版本号和用例数
            while (raf.getFilePointer() < raf.length()) {
                if (!ReplayContextHolder.isReplaying()) {
                    break;
                }
                int length = raf.readInt();
                byte[] traceBytes = new byte[length];
                for (int i = 0; i < length; i++) {
                    traceBytes[i] = raf.readByte();
                }
                FlowReplayTrace trace = (FlowReplayTrace) Hessian2Serializer.deserialize(traceBytes);
                try {
                    context.putTrace(trace);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
                if (Thread.currentThread().isInterrupted()) {
                    break;
                }
            }
        }
    }
}

==> spring-boot-ext-1.3.14.8-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/replay/ReplayTraceLoader.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay.replay;

import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayFileComponent;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.span.FlowReplayTrace;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.event.ReplayEndEvent;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.event.ReplayStartEvent;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.serializer.Hessian2Serializer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * 下载并解压录制的用例集文件，然后把用例读取到内存中，供用例消费者消费
 * Created by guoyanfei .
 * 2019-02-26 .
 */
@Slf4j
public class ReplayTraceLoader {

    @Resource
    private FlowReplayFileComponent flowReplayFileComponent;

    private volatile Thread usecaseSetLoadThread = null;

    /**
     * 监听 "回归开始" 事件，加载用例集文件内容到内存中
     * @param event
     */
    @EventListener(ReplayStartEvent.class)
    public void replayStartEventListener(ReplayStartEvent event) {
        usecaseSetLoadThread = new Thread(new Runnable() {

            @Override
            public void run() {
                ReplayContext context = ReplayContextHolder.getReplayContext();
                if (context == null) {
                    return;
                }
                try {
                    loadUsecaseSet(context);
                    // 用例集加载完成
                    context.tracesLoadFinish();
                } catch (Exception e) {
                    log.error("引流回归_用例集加载异常_reportId_{}", context.getReportId(), e);
                    ReplayContextHolder.forceEnd();
                }
            }
        }, "DBThread-Replay-加载用例集");

        usecaseSetLoadThread.start();
    }

    @EventListener(ReplayEndEvent.class)
    public void replayEndEventListener(ReplayEndEvent event) {
        if (usecaseSetLoadThread != null) {
            usecaseSetLoadThread.interrupt();
        }
    }

    /**
     * 加载用例集
     * @throws IOException
     */
    private void loadUsecaseSet(ReplayContext context) throws IOException {
        String localReplayPath = flowReplayFileComponent.download(context.getUsecaseSetUrl());
        File file = new File(localReplayPath);
        try (RandomAccessFile raf = new RandomAccessFile(file, "rw")) {
            raf.skipBytes(8);   // 跳过版本号和用例数
            while (raf.getFilePointer() < raf.length()) {
                if (!ReplayContextHolder.isReplaying()) {
                    break;
                }
                int length = raf.readInt();
                byte[] traceBytes = new byte[length];
                for (int i = 0; i < length; i++) {
                    traceBytes[i] = raf.readByte();
                }
                FlowReplayTrace trace = (FlowReplayTrace) Hessian2Serializer.deserialize(traceBytes);
                try {
                    context.putTrace(trace);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
                if (Thread.currentThread().isInterrupted()) {
                    break;
                }
            }
        }
    }
}
